#
# Licence GPL v3.
#
# Copyright Chris Johns (chrisj@rtems.org)
#
import pq
import optparse
import os
import re
import sys
import tailer

def build_status(base, data, workspace, source, targets, show):
    if show:
        for target in targets:
            try:
                print " %s %s: %s" % (pq.target_name(target, 8), 
                                      pq.status(workspace, target),
                                      pq.stats(base, data, workspace,
                                               source, target))
            except IOError, e:
                pass
                
def queue_running(base, data, workspace, source, targets, running, results):
    state = tailer.tail(running, 1)
    if len(state) != 1:
        print >> sys.stderr, "Unknown queue state: invalid running"
        return
    if state[0] == 'cleaning':
        print 'Cleaning old source and build files.'
    elif state[0] == 'source':
        print 'Getting source code'
    elif state[0] == 'bootstrap':
        print 'RTEMS bootstrap'
    elif state[0].startswith('target:'):
        # Look backward until we find a make directory
        target = state[0].split(':')[1]
        try:
            stdout = open(os.path.join(workspace, target, 'stdout.txt'))
            lines = 100
            while lines < 30000:
                output = tailer.tail(stdout, lines)
                found = False
                for o in range(len(output) - 1, 0, -1):
                    if output[o].find('Entering directory') > 0:
                        path = os.path.abspath(re.compile(r"[`']").split(output[o])[1])
                        path = path.replace(os.path.join(workspace, target, ''), '')
                        print '%s: %s' % (pq.target_name(target, 3), path)
                        found = True
                        break
                if found:
                    break
                lines = lines * 4
            stdout.close()
        except IOError, e:
            print target, "waiting for output"
    else:
        print >> sys.stderr, "Unknown queue state:", state[0]
    build_status(base, data, workspace, source, targets, results)

def queue_idle(base, data, workspace, source, targets, status):
    print "Finished"
    build_status(base, data, workspace, source, targets, status)
    
#
# Command line options.
#
cl_options = [
    optparse.make_option('-r', action = 'store_true', dest = 'results',
                         default = False,
                         help = 'Display the results'),
    # Standard options
    optparse.make_option('-d', action = 'store', type = 'string', 
                         dest = 'data', default = os.environ['HOME'],
                         help = 'Specify the data directory'),
    optparse.make_option('-w', action = 'store', type = 'string', 
                         dest = 'workspace', default = None,
                         help = 'Specify the workspace directory'),
    optparse.make_option('-s', action = 'store', type = 'string', 
                         dest = 'source', default = None,
                         help = 'Specify the source directory'),
    optparse.make_option('-t', action = 'store', type = 'string', 
                         dest = 'targets', default = None,
                         help = 'List of comma seperated targets'),
    optparse.make_option('-c', action = 'store', type = 'string',
                         dest = 'config',
                         default = os.environ['HOME'] + '/.pq-config',
                         help = 'Configration file name to read')
]

def user_main(base, argv):
    (options, config, data, workspace, source, targets) = \
        pq.parse_std_args(argv, cl_options)
    try:
        running = open(workspace + '/running')
        queue_running(base, data, workspace, source, targets, running, options.results)
        running.close()
    except IOError, e:
        running = None
    if not running:
        queue_idle(base, data, workspace, source, targets, options.results)
